/* File:        $Id: FileListStatus.java 1338 2010-03-17 15:27:53Z svc $
 * Revision:    $Revision: 1338 $
 * Author:      $Author: svc $
 * Date:        $Date: 2010-03-17 16:27:53 +0100 (Wed, 17 Mar 2010) $
 *
 * The Netarchive Suite - Software to harvest and preserve websites
 * Copyright 2004-2010 Det Kongelige Bibliotek and Statsbiblioteket, Denmark
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 
 *  USA
 */
package dk.netarkivet.archive.arcrepositoryadmin;

import dk.netarkivet.common.exceptions.ArgumentNotValid;

/**
 * The status for the file list updates.
 * This is used by the DatabaseBasedActiveBitPreservation.
 */
public enum FileListStatus {
    /** If the status has not been defined. This is the initial value.*/
    NO_FILELIST_STATUS,
    /** If the file is missing from a file list or a checksum list.*/
    MISSING,
    /** If the file has the correct checksum.*/
    OK;
    
    /**
     * Method to retrieve the FileListStatus based on an integer.
     *  
     * @param status A specific integer for the upload status
     * @return The UploadStatus related to the certain integer
     * @throws ArgumentNotValid If argument rt does not correspond
     * to a UploadStatus.
     */
    public static FileListStatus fromOrdinal(int status) 
            throws ArgumentNotValid {
        switch (status) {
            case 0: return NO_FILELIST_STATUS;
            case 1: return MISSING;
            case 2: return OK;
            default: throw new ArgumentNotValid(
                    "Invalid filelist status with number " + status);
        }
    }
}
